#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2010-2018. All rights reserved.
# BepKit version 2.1.T1
#Filename:bep_env.py(python3.7)
#Function:load bep_env
import argparse
import os
import sys
import time
import platform
import errno
import shutil
import subprocess
import datetime
import fileinput
import logging

try:
    import configparser
except ImportError:
    import ConfigParser as configparser
BEP_LOG_FORMAT = "%(asctime)s %(levelname)s %(name)s %(message)s"
BEP_DATE_FORMAT = "%Y-%m-%d_%H:%M:%S"
logging.basicConfig(level = logging.INFO, format=BEP_LOG_FORMAT, datefmt=BEP_DATE_FORMAT)
bep_logger = logging.getLogger("bep_env")

BEP_VERSION="2.1.T1"
BEP_LIB_SOFT_LINK="libbep_env.so"
BEP_LIB_REAL_NAME="".join([BEP_LIB_SOFT_LINK, ".", BEP_VERSION])

class GVar(object):
    BEP_TRACE_ARG = False

def bep_get_install_dir():
    p = subprocess.Popen(['which', 'bep_env.py'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    script_softlink, err = p.communicate()
    if isinstance(script_softlink, bytes):
        script_softlink = script_softlink.decode()
    script_softlink = script_softlink.replace('\n', '')
    if os.path.islink(script_softlink):
        return os.path.dirname(os.readlink(script_softlink))
    else:
        return os.path.dirname(os.path.realpath(script_softlink))

def bep_version():
    """get bepkit version infomation"""
    bep_install_dir = bep_get_install_dir()
    lib64 = "".join([bep_install_dir, "/bep_env/64/", BEP_LIB_REAL_NAME])
    lib32 = "".join([bep_install_dir, "/bep_env/32/", BEP_LIB_REAL_NAME])
    if os.path.exists(lib32):
        lib_version = os.popen('strings %s|grep "BepKit version"'%lib32)
        bep_logger.info(lib_version.read().replace('\n', '').replace('\r', ''))
    lib64_version = os.popen('strings %s|grep "BepKit version"'%lib64)
    bep_logger.info(lib64_version.read().replace('\n', '').replace('\r', ''))

def bep_refresh_timestamp(conf, section):
    """refresh timestamp in bep_env.conf"""
    try:
        config = configparser.ConfigParser()
        config.read(conf)
        if section not in config.sections():
            bep_logger.error("%s not in config, please check!" % section)
        else:
            flag = 0
            current_time = datetime.datetime.now()
            for line in fileinput.input(conf, inplace=1):
                current_time = current_time - datetime.timedelta(seconds=20)
                line = line.replace('\n', '').replace('\r', '')
                if line[1:10] == section:
                    print(line)
                    flag = 1
                    continue
                if line[0:1] == '#' or line[0:1] == '[':
                    print(line)
                    flag = 0
                    continue
                if flag == 1:
                    key_name = line.split('=')[0]
                    if key_name.strip():
                        out_line = "%s=%s" % (key_name, current_time.strftime('%Y-%m-%d,%H:%M:%S'))
                        print(out_line)
                else:
                    print(line)
            bep_logger.info("The timestamp in %s has been refreshed." % conf)
    except Exception as err:
        bep_logger.error(str(err))

def is_valid_datetime(DateTime):
   '''
   funcntion description: validate time stamp string format
   Globals:	None
   Arguments:	time stamp string
   Returns:	True or False
   '''
   try:
       if ":" in DateTime:
           time.strptime(DateTime, "%Y-%m-%d,%H:%M:%S")
           return True
   except:
       return False

   return False


def bep_check_and_read_timestamp(conf, section):
    """check timestamp in bep_env.conf"""
    try:
        config = configparser.ConfigParser()
        config.read(conf)
        if section not in config.sections():
            bep_logger.error("%s not in config, please check!" % section)
        else:
            if not config.options(section):
                bep_logger.error("%s is empty, please check!" % section)
            for option in config.options(section):
                    ini_line_time = config.get(section, option)
                    isVaild = is_valid_datetime(ini_line_time)
                    if not isVaild:
                       logging.error("[-rb]: {} [timestamp] {} is invalid: {}".format(conf, option, ini_line_time))
                    else:
                        msg = "".join([option, " is :", config.get(section, option), ])
                        bep_logger.info(msg)
    except Exception as err:
        bep_logger.error(str(err))

def bep_enable_trace():
    bep_install_dir = bep_get_install_dir()
    db_file = os.path.join(bep_install_dir,'sqlite3/bepkit_trace.db')
    db_default_name = 'bepkit_trace.db'
    if not os.path.exists(db_file):
        bep_logger.error("can't find {}".format(db_default_name))
        exit()
    db_target = os.environ.get('BEP_TRACE_OUTPUT')

    if not db_target:
        bep_logger.error("BEP_TRACE_OUTPUT should be set if enable bep_trace.")
        return
    if db_target and not os.path.isabs(db_target):
        bep_logger.error('BEP_TRACE_OUTPUT must be absulte path not relative path')
        return
    if not os.path.exists(db_target):
        shutil.copyfile(db_file, db_target)
    bep_logger.info("Init bep sqlite database {}".format(db_target))

def bep_deploy_env(conf, keytime, section):
    """deploy bep environment"""
    # Step 1: read bep_env.conf
    config = configparser.ConfigParser()
    config.read(conf)
    if not config.has_section(section):
        bep_logger.error("section %s not in config file, please check!" %section)
        exit()
    else:
        if not config.has_option(section, keytime):
            bep_logger.warn("%s does not have %s, and will get the value of bep_timestamp" % (section, keytime))
            keytime = "bep_timestamp"
    saved_section_name = config.sections()
    BEP_EXPORT_SECTION = ["user_define_data", "trace_define_data", "bep_time_blacklist", "whitelist", "blacklist", "hash_seed", "timezone", "locale", "sysinfo"]
    # Step 2: set bepkit environment varible
    for cur_section_name in saved_section_name:
        if cur_section_name in BEP_EXPORT_SECTION:
            saved_key_name = config.options(cur_section_name)
            for cur_key_name in saved_key_name:
                cur_key_value = config.get(cur_section_name, cur_key_name)
                if cur_key_value:
                    os.environ[cur_key_name.upper()] = cur_key_value
    if not os.getenv('BEP_UID') or not os.getenv('BEP_GID'):
         os.unsetenv('BEP_UIDLIST')
    os.environ["BEP_TIME"] = config.get(section, keytime)
    if os.getenv('SOURCE_DATE_EPOCH'):
        os.environ['SOURCE_DATE_EPOCH'] = \
            os.popen('LD_PRELOAD=libbep_env.so TZ=Asia/Shanghai BEP_CGTLIST=date date +%s').read().replace('\n', '')
    # Step 3: enable bep_trace
    if (os.environ.get('BEP_TRACE_ENABLE') and os.environ['BEP_TRACE_ENABLE'] == 'yes') or GVar.BEP_TRACE_ARG:
        bep_logger.info("Enable Bepkit Trace")
        bep_enable_trace()

    # Step 4: enable bepkit
    if 'LD_PRELOAD' not in os.environ:
        os.environ['LD_PRELOAD'] = BEP_LIB_SOFT_LINK
    elif BEP_LIB_SOFT_LINK in os.environ['LD_PRELOAD']:
        bep_logger.info("it has been set bep_env environment")
    else:
        os.environ['LD_PRELOAD'] = ":".join([os.environ['LD_PRELOAD'], BEP_LIB_SOFT_LINK])
    bep_logger.info("LD_PRELOAD is %s " % os.getenv('LD_PRELOAD'))
    bep_logger.info("BEP_TIME is %s, the value of %s." % (os.getenv('BEP_TIME'), keytime))
    # Step 5: Check whether the bep environment is deployed or not
    my_env = os.environ.copy()
    my_env['BEP_CGTLIST']='date'
    my_env['BEP_GTDLIST']='date'
    date1 = subprocess.check_output(['date', '+%s'], env=my_env)
    date_time = '--date={}'.format(os.environ.get('BEP_TIME').replace(',',' '))
    date2 = subprocess.check_output(['date', date_time, '+%s'])
    if date1 == date2:
        bep_logger.info("Set BEP_ENV successful!")
    else:
        bep_logger.error("Set BEP_ENV fail!")
    return 0

def bep_show_help():
    usage = '''
            bep_env.py -s  [CONF] [KEY_TIME] [SECTION]
        or: bep_env.py -c  [CONF] [SECTION]
        or: bep_env.py -rb [CONF] [SECTION]
        or: bep_env.py -v
        or: bep_env.py -h

        Options:
            -s  deploy bep environment with the related variables in bep_env.conf
            -c  refresh the value of timestamp in bep_env.conf with the latest time.
            -rb check and read the value of timestamp in bep_env.conf.
            -h  display this help messages and exit.
            -v  display version messages and exit.
            -t  enable beptrace.(must define BEP_TRACE_OUTPUT in config file.)
            [CONF] specify the CONF file. if not set, search bep_env.conf in the current path.
            [KEY_TIME] specify the name of key. if not set, the name is bep_timestamp.
            [SECTION] specify the name of time section. if not set, the name is timestamp.

        CONF:
            [Section]
            key=value

    '''
    bep_logger.info(usage)

def bep_install():
    """For compatibility"""
    pass

def bep_remove():
    """For compatibility"""
    pass


def setBepkit(option, *args):
    bep_logger.info("Call by import mode")
    conf = "./bep_env.conf"
    section = "timestamp"
    keytime = "bep_timestamp"

    if option == "-t":
        GVar.BEP_TRACE_ARG = True
        option = args[0]
        args = args[1:]

    length = len(args)
    if length == 0:
        pass
    elif length == 1:
        conf = args[0]
    elif length == 2:
        conf = args[0]
        section = args[1]
    elif length == 3:
        conf = args[0]
        keytime = args[1]
        section = args[2]
    else:
        bep_logger.error("The length of parameters is incorrect.")

    if option == "-s":
        bep_deploy_env(conf, keytime, section)
    elif option == "-c":
        bep_refresh_timestamp(conf, section)
    elif option == "-rb":
        bep_check_and_read_timestamp(conf, section)
    elif option == "-v":
        bep_version()
    elif option == "-h":
        bep_show_help()
    elif option == "-i":
        bep_install()
    elif option == "-e":
        bep_remove()
    else:
        bep_logger.error("The option is incorrect.")
        bep_show_help()

def bepStart():
    PARSER = argparse.ArgumentParser(add_help=False)
    PARSER.add_argument("-h", "--bep_show_help", action="store_true")
    PARSER.add_argument("-v", "--bep_version", action="store_true")
    PARSER.add_argument("-c", "--bep_refresh_timestamp", action="store_true")
    PARSER.add_argument("-rb", "--bep_check_and_read_timestamp", action="store_true")
    PARSER.add_argument("-s", "--bep_deploy_env", action="store_true")
    PARSER.add_argument("-i", "--bep_install", action="store_true")
    PARSER.add_argument("-e", "--bep_remove", action="store_true")
    PARSER.add_argument("-t", "--bep_trace_enable", action="store_true")
    PARSER.add_argument("CONF", nargs="?", default="")
    PARSER.add_argument("KEY_TIME", nargs="?", default="")
    PARSER.add_argument("SECTION", nargs="?", default="")

    args = PARSER.parse_args()
    args = args.__dict__

    if args["bep_trace_enable"]:
        GVar.BEP_TRACE_ARG = True

    if args["CONF"]:
        conf = ''.join([args["CONF"]])
        if not os.path.exists(conf):
            bep_logger.error("%s not exist" %conf)
            exit(0)
        bep_logger.info("load the specified config file %s" %conf)
    else:
        bep_env_path = sys.path[0]
        bep_logger.info("bep_env.py is in %s " % bep_env_path)
        conf = ''.join([bep_env_path, "/bep_env.conf"])
        if not os.path.exists(conf):
            bep_logger.error("%s not exist" %conf)
            exit(0)
        bep_logger.info("load the default config file %s" %conf)
    if args["KEY_TIME"]:
        keytime = "".join(args["KEY_TIME"])
    else:
        keytime = "bep_timestamp"
    if args["SECTION"]:
        section = "".join(args["SECTION"])
    else:
        section = "timestamp"

    if args["bep_deploy_env"]:
        bep_deploy_env(conf, keytime, section)
    elif args["bep_refresh_timestamp"]:
        bep_refresh_timestamp(conf, section)
    elif args["bep_check_and_read_timestamp"]:
        bep_check_and_read_timestamp(conf, section)
    elif args["bep_version"]:
        bep_version()
    elif args["bep_install"]:
        bep_install()
    elif args["bep_remove"]:
        bep_remove()
    elif args["bep_show_help"]:
        bep_show_help()
    else:
        bep_logger.error("The option is incorrect.")
        bep_show_help()

if __name__ == '__main__':
    bep_logger.info("Call by exec mode or directly")
    bepStart()
else:
    pass

__all__ = ['setBepkit']
